/***********************************************************************************
************************************************************************************

PIC10(L)F322 Development Board (AC103011-P) demo

Copyright (C)2012 Giovanni Bernardo
http://www.settorezero.com
date:  05/03/12

MCU:					Microchip PIC10F322
Clock:					16MHz Internal
Compiler:				HI-TECH C Compiler for PIC10/12/16 MCUs Version 9.83

You may NOT distribute or sell this code without explicit authorization

THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
WE ARE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
(INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.

************************************************************************************
************************************************************************************/

#include <htc.h>
#define _XTAL_FREQ	16000000UL

#define	SW1		PORTA3
#define LED1	LATA1
#define LED2	LATA0
// trimmer on 	PORTA2

unsigned char ledcounter=250; // blinking counter for led1
unsigned char advalue; // value from AD conversion
unsigned char stopflash=0; // flag used to stop led blinking

// internal oscillator used
__CONFIG(FOSC_INTOSC & BOREN_OFF & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & CP_OFF & LVP_OFF & LPBOR_OFF & BORV_LO & WRT_OFF);

void main(void)
	{
	OSCCON=0b01110000; // internal oscillator @ 16MHz
	while (!HFIOFS); // wait for internal oscillator stability
	
	TRISA =0b00001101; // A1 output. A2,A3 inputs (A2=analog). A0 input for now, will become output later, after PWM settings
	ANSELA=0b00000100; // only A2 as analog input
	ADCON =0b11101001; // adc enabled, read from AN2, clock=frc
	
	// TOsc=62.5nS (FOsc=16MHz)
	// prescaler 1:64 -> tmr0 increments by 1 every 62.5*4*64 = 16uS
	OPTION_REG=0b10000101; // prescaler 1:64 assigned to timer0
	TMR0=256-62; // 16*62 = 992uS. Overflow interrupt will fire every 992uS
	
	// PWM module setting (See datasheet, pag. 110)
	PWM1CON=0; // reset PWM1 control register
	PR2=0x80; // pwm period = 32.25uS (PWM frequency=31KHz). See datasheet, pag.108
	PWM1DCH=0; // reset PWM1 duty cycle register H
	PWM1DCL=0; // reset PWM1 duty cycle register L
	TMR2IF=0; // timer2 interrupt flag reset
	T2CON=0b00000000; // timer2 prescaler 1:1
	TMR2ON=1; // enable timer2
	PWM1OE=1; // enable PWM1 output
	while (!TMR2IF); // wait until timer2 overflows
	PWM1EN=1; // enable PWM1 module
	TRISA &= 0b11111110; // enable only PORTA0 (PWM1 out) as output
	
	GIE=1; // global interrupt enable
	TMR0IE=1; // timer0 interrupt enable
	
	while (1)
		{
		GO_nDONE=1; // start AD conversion
		while(GO_nDONE); // wait until conversion complete
		advalue=ADRES; // AD conversion result
		PWM1DCH=advalue; // assign AD result to PWM1 duty cycle register
						
		if (!SW1) // SW1 pressed?
			{
			__delay_ms(150); //antibounce delay
			if(!SW1) // still pressed?
				{
				stopflash ^= 1; // enable/disable D1 blinking
				if (stopflash) // blinking disabled?
					{
					LED1=0; // led off
					ledcounter=250; // reset blink counter
					}
				}
			} // \sw1 pressed
		
		} // \while(1)
	
	} // \main

// interrupt every 992uS
void interrupt ISR(void)
	{
	TMR0IF=0; // reset timer0 interrupt flag
	if (!stopflash) // blinking enabled?
		{
		ledcounter--; // decrement led blink counter
		if (ledcounter==0)	
			{
			LED1^=1; // invert led status
			ledcounter=250; // reset blink counter
			}
		}
	TMR0=256-62; // reload timer0
	} // \isr
